% MATLAB SIMULATION OF NSA FS-1016 CELP v3.2
% COPYRIGHT (C) 1995-99 ANDREAS SPANIAS AND TED PAINTER
%
% This Copyright applies only to this particular MATLAB implementation
% of the FS-1016 CELP coder.  The MATLAB software is intended only for educational
% purposes.  No other use is intended or authorized.  This is not a public
% domain program and distribution to individuals or networks is strictly
% prohibited.  Be aware that use of the standard in any form is goverened
% by rules of the US DoD.  Therefore patents and royalties may apply to
% authors, companies, or committees associated with this standard, FS-1016.  For
% questions regarding the MATLAB implementation please contact Andreas
% Spanias at  (602) 965-1837.  For questions on rules,
% royalties, or patents associated with the standard, please contact the DoD.
%
% ALL DERIVATIVE WORKS MUST INCLUDE THIS COPYRIGHT NOTICE.
%
% ******************************************************************
% CELPSYN
%
% NSA CELP 3.2a SYNTHESIS DRIVER
%
% PORTED TO MATLAB FROM CELP 3.2a C RELEASE
% 8-2-94
%
% ******************************************************************
%
% DESCRIPTION
%
% CELP 3.2a synthesis driver
%
% DESIGN NOTES
%
% This driver has been derived from the main loop of the NSA CELP 3.2a
% simulation.  CELP synthesis accepts a CELP analysis output bitstream
% (144 bits/frame or 4.8 kbps) as input and produces three (3) 16-bit
% PCM speech data files as output (binary):
%
%    1) *.NPF  -  Non-postfiltered synthetic output speech
%    2) *.SPO  -  Postfiltered synthetic output speech
%    3) *.HPF  -  Highpass filtered synthetic output speech
%
% This synthesis driver also includes bit error rate simulation
%
% VARIABLES
%
% INTERNALS
%   SimType       -    Simulation type - analysis/synthesis or synthesis only
%   pstream       -    Permuted version of CELP analysis output bitstream
%   bitperm       -    Bit stream permutation table
%   stream        -    CELP analysis output bitstream
%   line          -    Frame buffer, hex bitstream CELP analysis output,
%                      obtained from hex file 'ofile.cha'
%   pointer       -    CELP analysis output bitstream current bit pointer
%   ber           -    Simulated bit error rate
%   mask          -    Bitstream mask, used to protect specific bits from bit errors
%   errors        -    Cumulative sum over all frames of number of bit errors
%   total         -    Total number of bits processed
%   protect       -    Hamming bit protection enable/disable flag
%   codeword      -    Hamming (15,11) codeword, formed from protected bits
%   bitprotect    -    Bitstream indicies of bits protected by the hamming code
%   twoerror      -    Flag indicating two errors in current received frame
%   hmatrix       -    Hamming matrix, used during hamming codeword decoding
%   syndrometable -    Hamming syndrome table  "        "                 "
%   syndrome      -    Parity error flag, generated by hamming decoder
%   paritybit     -    Extraneous paritybit used for codeword parity overall
%   syndavg       -    Smoothed error rate
%   findex        -    LSP quantization table indicies, unpacked from bitstream
%   sbits         -    Bit allocation, LSP quantization
%   no            -    LPC synthesis filter order
%   newfreq       -    Decoded LSPs, derived from bitstream and quant table
%   lsp           -    Set of linearly interpolated lsps for nn subframes of current frame
%   nn            -    Number of subframes
%   bitpointer    -    Temporary copy of pointer, bitstream pointer
%   taus          -    Pitch lag values (nn) decoded from the bitstream
%   pgs           -    Pitch lag gain values (nn) decoded from the bitstream
%   cbi           -    Stochastic codebook indicies (nn) decoded from bitstream
%   cbg           -    Stochastic codebook gains (nn) decoded from bitstream
%   pbits         -    Bit allocation, adaptive codebook gain and index quantization
%   cbbits        -    Bit allocation, stochastic codebook index
%   cbgbits       -    Bit allocation, stochastic codebook gain
%   bitsum1       -    Bit allocation sum used during parameter decoding
%   bitsum2       -    Bit allocation sum used during parameter decoding
%   pddecode      -    Tau (pitch lag) permutation table
%   pdtabi        -    Tau delay table index
%   subframe      -    Current subframe under analysis (1:nn)
%   l             -    Subframe length, in samples
%   vl, vh        -    Subframe limits within current frame buffer
%   cbindex       -    Stochastic codebook index, current subframe
%   decodedgain   -    Stochastic codebook gain, current subframe
%   vdecoded      -    Synthesis excitation vector; combination of both codebook contributions
%   bb            -    Pitch lag (adaptive codebook) index and gain, current subframe
%   dps           -    Buffer used during adaptive codebook vector formation
%   idb           -    Dimension of buffer used in adaptive codebook vector formation (see PITCHVQ.M)
%   fci           -    LPC synthesis filter coefficients for current subframe
%   dss           -    LPC synthesis filter memory
%   descale       -    Output speech scale factor
%   minv          -    Minimum allowed output speech sample value
%   maxv          -    Maximum allowed output speech sample value
%   npf           -    Non-postfiltered output speech (1 frame)
%   pf            -    Postfiltered output speech (1 frame)
%   hppf          -    Highpass filtered output speech (1 frame)
%   fpo           -    Output file pointer, post-filtered
%   fpnpf         -    Output file pointer, non-post filtered
%   fphpf         -    Output file pointer, highpass filtered
%   junkbuf       -    Unused dummy variable buffer (diskio return)
%   status        -    Status returned by file write operation (diskio return)
%   sumsnr        -    SNR averaging summation, non-postfiltered output speech
%   framesnr      -    SNR averaging subframe count, non-postfiltered output speech
%   snrflag       -    SNR threshold flag, non-postfiltered output speech
%   sumsnrp       -    SNR averaging summation, postfiltered output speech
%   framesnrp     -    SNR averaging subframe count, postfiltered output speech
%   snrflagp      -    SNR threshold flag, postfiltered output speech
%   sumsnrh       -    SNR averaging summation, highpass filtered output speech
%   framesnrh     -    SNR averaging subframe count, highpass filtered output speech
%   snrflagh      -    SNR threshold flag, highpass filtered output speech
%   dummy1        -    Unused dummy variable 
%   dummy2        -    Unused dummy variable
%   dummy3        -    Unused dummy variable
%   framedm       -    Speech distortion averaging subframe count
%   sumdm         -    Speech distortion summation over framedm subframes
%   dm            -    Speech distortion measures, current subframe
%   ssub          -    Subframe analysis buffer (input speech, ll samples = 1/2 old, 1/2 new)
%   hamws         -    Hamming window, one full subframe in length
%   ip            -    Post filter estimate of input power
%   op            -    Post filter estimate of output power
%   dp1           -    Postfilter memory
%   dp2           -    Postfilter memory
%   dp3           -    Postfilter memory
%   dhpf1o        -    FIR (all zero) highpass filter memory
%   dhpf2o        -    IIR (all pole) highpass filter memory
%   ahpfo         -    Highpass filter H(Z) numerator coefficients
%   bhpfo         -    Highpass filter H(Z) denominator coefficients
%
%
% GLOBALS
%   FrameCnt      -    Current frame number
%   guiSin        -    Input speech signal
%   guiSout       -    Output speech signal
%   guiRCs        -    LPC synthesis filter coefficients
%   guiFDM        -    Speech distortion averaging frame count
%   guisdm        -    Speech distortion summation over guiFDM frames
%   guidm         -    Speech distortion measures, current frame
%   avgSnrNPF     -    Average SNR, non-postfiltered output speech
%   avgSnrPF      -    Average SNR, postfiltered output speech
%   avgSnrHPF     -    Average SNR, highpass filtered output speech
%   SnrNPF        -    Current frame SNR, non-postfiltered output speech
%   SnrPF         -    Current frame SNR, postfiltered output speech
%   SnrHPF        -    Current frame SNR, highpass filtered output speech
%   guiExcit      -    Synthesis filter excitation
%   guiState      -    Graphical output or status output mode
%                      0 = status mode (non-interactive), 1 = graphics mode
%
% CONSTANTS
%   LFRAME        -   Analysis frame size
%   READ          -   Read flag to control diskio
%   WRITE         -   Write flag to control diskio
%   EOF           -   Status flag for diskio
%   STREAMBITS    -   Bits per frame of CELP analysis output
%   SYNTHESIS     -   Synthesis only simulation control flag
%   CODELENGTH1   -   Hamming codeword overall length (in bits)
%   CODELENGTH2   -   Hamming codeword data length (in bits)
%   PARITYLENGTH  -   Hamming codeword error protection length (in bits)
%   SYNDRUN       -   Bit error rate averaging constant
%   ALPHA         -   Postfilter pole/zero bandwidth expansion parameter
%   BETA          -   Postfilter pole/zero bandwidth expansion parameter
%
% ******************************************************************

% DECLARE GLOBAL CONSTANTS
global EOF LFRAME READ WRITE;

% DECLARE GLOBAL VARIABLES
global guiSin         guiSout    guiRCs    guiState
global guidm          guisdm     guiFDM
global avgSnrNPF      avgSnrPF   avgSnrHPF guiExcit
global SnrNPF         SnrPF      SnrHPF

% READ CELP ANALYSIS OUTPUT FROM A HEX FILE FOR SYNTHESIS ONLY SIMULATIONS
if SimType == SYNTHESIS
    pstream = gethex( STREAMBITS, line );
end

% INVERT BIT STREAM PERMUTATION IMPLEMENTED AT TRANSMITTER
stream( bitperm ) = pstream;

% SET MASK BITS TO PROTECT SPECIFIC BITS FROM BIT ERRORS
% mask(1:2:144) = ones(72,1);

% SIMULATE TRANSMISSION OF CELP BIT STREAM AND INJECT ERRORS
% AT SPECIFIED BIT ERROR RATE (BER)
% SIMULATE LOSSY CHANNEL BY INJECTING BIT ERRORS
[ stream, errors, total ] = biterror( ber, mask, stream, STREAMBITS, errors, total );

% DECODE ERROR PROTECTION BITS
if protect == TRUE

    % ADJUST BIT STREAM POINTER AND UNPACK PARITY BITS
    if SimType == SYNTHESIS
        pointer = 139;
    else
        pointer = pointer - PARITYLENGTH - 2;
    end
    [ codeword( CODELENGTH2+1:CODELENGTH1 ), pointer ] = ...
    unpack( stream, [1,1,1,1], pointer );

    % EXTRACT CODEWORD FROM STREAM ARRAY
    codeword( 1:CODELENGTH2 ) = stream( bitprotect );

    % REPACK EXPANSION BIT (BISNU BIT)
    codeword( 11 ) = 0;

    % DECODE HAMMING ERROR PROTECTION
    [ codeword, twoerror, syndrome ] = ...
    decodham( CODELENGTH1, hmatrix, syndrometable, paritybit, codeword );

    % DISABLE PARITY CHECK
    twoerror = FALSE;

    % ESTIMATE BIT ERROR RATE - RUNNING AVERAGE OF BAD SYNDROMES
    if syndrome ~= 0
        syndrome = 1;
    end
    syndavg = ( 1.0 - (1.0/SYNDRUN) ) * syndavg + ( 1.0 / SYNDRUN ) * syndrome;

    % REPACK ERROR CORRECTED, PROTECTED BITS
    stream( bitprotect ) = codeword( 1:CODELENGTH2 );

    % FRAME REPEAT IF TWO ERRORS HAVE BEEN DETECTED IN THE CODEWORD
    % THIS FEATURE IS DISABLED FOR TRUE FS1016 SIMULATION, WHICH CAN ONLY
    % DETECT AND CORRECT SINGLE BIT ERRORS GIVEN THE HAMMING(11,15)
    % PROTECTION SCHEME. THE ADDITIONAL PARITYBIT VARIABLE CAN BE USED
    % TO DETECT 2 ERRORS, HOWEVER IT IS OUTSIDE OF THE FS1016 DEFINED
    % BITSTREAM, SO IT HAS BEEN DISABLED BY DEFAULT HERE.
    % SEE FALSE CONSTANT FOR TWOERROR, ABOVE.
    if twoerror == TRUE
        fprintf( 'celpexec: two errors have occured in frame %d\n', FrameCnt );
    end
end

% DECODE CELP SYNTHESIS PARAMETERS FROM THE BIT STREAM, INCLUDING
% LSPs, CODEBOOK INDICIES, AND CODEBOOK GAINS

% UNPACK LSPS FROM DATA STREAM AND DECODE
pointer = 0;
[ findex, pointer ] = unpack( stream, sbits, pointer );
newfreq = lspdecod( findex, no );

% INTERPOLATE LSPS FOR NN SUBFRAMES
lsp = intsynth( newfreq, nn, no, twoerror, syndavg );

% DECODE CODEBOOK AND PITCH PARAMETERS TO SET UP FOR SYNTHESIS
bitpointer = pointer;
[ taus, bitpointer ] = dcodtau( pbits(1), pbits(2), bitsum1, bitsum2, ...
                                bitpointer, nn, stream, pddecode, pdtabi );
[ pgs, bitpointer ] = dcodpg( pbits(3), bitsum1, bitsum2, bitpointer, ...
                              nn, stream );
[ cbi, bitpointer ] = dcodcbi( cbbits, bitsum1, bitsum2, bitpointer, ...
                               nn, stream );
[ cbg, bitpointer ] = dcodcbg( cbgbits, bitsum1, bitsum2, bitpointer, ...
                               nn, stream );

% ADJUST SUBFRAME COUNTER FOR ANALYSIS-SYNTHESIS SIMULATIONS BECAUSE
% NSEG HAS BEEN RUN UP BY NN DURING ANALYSIS
if SimType == ANALYSIS
    nseg = nseg - nn;
end

% SYNTHESIZE EACH SUBFRAME
for subframe = 1:nn

    % UPDATE SUBFRAME COUNTER AND VECTOR INDICIES
    nseg = nseg + 1;
    vl = ((subframe-1)*l)+1; vh = vl+l-1;

    % GET STOCHASTIC CODEBOOK INDEX AND UNSMOOTHED GAIN
    cbindex = cbi(subframe);
    decodedgain = cbg(subframe);

    % RUN STOCHASTIC GAIN SMOOTHING IF BIT PROTECTION IS ENABLED
    if protect == TRUE
        decodedgain = smcbg( decodedgain, twoerror, syndavg, cbg, subframe );
    end

    % GENERATE STOCHASTIC EXCITATION VECTOR
    vdecoded(vl:vh) = vdecode( decodedgain, l, cbindex );

    % RUN PITCH LAG SMOOTHING IF BIT PROTECTION IS ENABLED
    if protect == TRUE
        taus(subframe) = smt( taus(subframe), twoerror, syndavg, taus(3), subframe );
    end
    bb(1) = taus(subframe);
    bb(3) = pgs(subframe);

    % RUN PITCH GAIN SMOOTHING IF BIT PROTECTION IS ENABLED
    if protect == TRUE
        bb(3) = smpg( bb(3), twoerror, syndavg, pgs, subframe );
    end

    % GENERATE PITCH EXCITATION VECTOR AND COMBINE WITH STOCHASTIC EXCITATION
    % TO PRODUCE COMPOSITE LPC EXCITATION VECTOR
    [ vdecoded(vl:vh), dps ] = pitchvq( vdecoded(vl:vh), l, dps, idb, bb, 'long' );
    guiExcit(vl:vh) = vdecoded(vl:vh);

    % CONVERT LSPs TO PCs, THEN RUN LPC SYNTHESIS, EXCITED BY VDECODED
    fci = lsptopc( lsp(subframe,:), no );
    [ dss, vdecoded(vl:vh) ] = polefilt( fci, no, dss, vdecoded(vl:vh), l );

    % SCALE AND CLAMP NON-POSTFILTERED DATA TO 16-BIT INTEGER RANGE
    vdecoded(vl:vh) = vdecoded(vl:vh) * descale;
    npf(vl:vh) = min( [ (vdecoded(vl:vh))'; maxv(vl:vh) ] )';
    npf(vl:vh) = round( max( [ (npf(vl:vh))'; minv(vl:vh) ] )' );

    % WRITE NON-POSTFILTERED OUTPUT TO SPEECH FILE
    [ junkbuf, status ] = diskio( WRITE, fpnpf, npf(vl:vh), l );
    if SimType == ANALYSIS

        % COMPUTE SEGMENTAL SNR AND UPDATE SNR AVERAGING ACCUMULATORS FOR
        % THE CURRENT SUBFRAME
        [ sumsnr, framesnr, snrflag, SnrNPF ] = segsnr( ssub(vl:vh), npf(vl:vh), ...
                                                sumsnr, framesnr );

        % COMPUTE DISTANCES/DISTORTIONS (LOG SPECTRAL ERROR, ETC.)
        if ( snrflag == TRUE )
            [ dm, sumdm, framedm ] = disto( ssub(vl:vh), npf(vl:vh), hamws, ...
                                            l, no, sumdm, framedm );
            guidm = dm;
            guisdm = sumdm;
            guiFDM = framedm;
            avgSnrNPF = 10*log10(sumsnr/framesnr);
        end
    end

    % DO POSTFILTERING
    [ vdecoded(vl:vh), ip, op, dp1, dp2, dp3 ] = ...
    postfilt( vdecoded(vl:vh), l, ALPHA, BETA, ip, op, dp1, dp2, dp3, fci, no );

    % TEST FOR CLIPPING IN OUTPUT SPEECH
    % IF CLIPPING IS DETECTED, SCALE SPEECH DOWN UNTIL CLIPPING IS ELIMINATED
    while clip( vdecoded( vl:vh ) ) == TRUE
        vdecoded( vl:vh ) = 0.05 * vdecoded( vl:vh );
        fprintf( 'celpsyn: Clipping detected @ frame %d\n', FrameCnt );
    end

    % CLAMP POSTFILTERED SPEECH TO 16-BIT INTEGER RANGE
    pf(vl:vh) = min( [ (vdecoded(vl:vh))'; maxv(vl:vh) ] )';
    pf(vl:vh) = round( max( [ (pf(vl:vh))'; minv(vl:vh) ] )' );

    % WRITE POSTFILTERED OUTPUT TO SPEECH FILE
    [ junkbuf, status ] = diskio( WRITE, fpo, pf(vl:vh), l );

    % HIGH PASS FILTER OUTPUT SPEECH
    [ dhpf1o, vdecoded(vl:vh) ] = zerofilt( ahpfo, 2, dhpf1o, vdecoded(vl:vh), l );
    [ dhpf2o, vdecoded(vl:vh) ] = polefilt( bhpfo, 2, dhpf2o, vdecoded(vl:vh), l );

    % CLAMP HIGHPASS FILTERED SPEECH TO 16-BIT INTEGER RANGE
    hppf(vl:vh) = min( [ (vdecoded(vl:vh))'; maxv(vl:vh) ] )';
    hppf(vl:vh) = round( max( [ (hppf(vl:vh))'; minv(vl:vh) ] )' );

    % WRITE HIGHPASS FILTERED OUTPUT TO SPEECH FILE
    [ junkbuf, status ] = diskio( WRITE, fphpf, hppf(vl:vh), l );

    % UPDATE AVERAGE SNR ACCUMULATORS
    if SimType == ANALYSIS

        % COMPUTE SEGMENTAL SNR AND UPDATE SNR AVERAGING ACCUMULATORS
        [ sumsnrp, framesnrp, snrflagp, dummy1 ] = segsnr( ssub(vl:vh), pf(vl:vh), ...
                                                   sumsnrp, framesnrp );
        [ sumsnrh, framesnrh, snrflagh, dummy1 ] = segsnr( ssub(vl:vh), hppf(vl:vh), ...
                                                   sumsnrh, framesnrh );

        % COMPUTE LOG SNRs
        if snrflagp == TRUE
            avgSnrPF = 10*log10(sumsnrp/framesnrp);
        end
        if snrflagh == TRUE
            avgSnrHPF = 10*log10(sumsnrh/framesnrh);
        end
    end
end

if SimType == ANALYSIS

    % COMPUTE OVERALL FRAME SNRs (FOUR SUBFRAMES)
    [ dummy1, dummy2, dummy3, SnrNPF ] = segsnr( ssub, npf, 0, 0 );
    [ dummy1, dummy2, dummy3, SnrPF ] = segsnr( ssub, pf, 0, 0 );
    [ dummy1, dummy2, dummy3, SnrHPF ] = segsnr( ssub, hppf, 0, 0 );
end

% GENERATE GRAPHICAL OUTPUTS; PASS VALUES VIA GLOBALS
if guiState == 1
    guiRCs = fci;
    guiSin = ssub;
    guiSout = [ npf, pf, hppf ];
%     gui;

    % MUST BREAK OUT OF WHILE SINCE GUI CALLBACK DRIVES SIMULATION
    % WHILE LOOP IS USED ONLY FOR STATUS MODE
%     break;
end

