% MATLAB SIMULATION OF NSA FS-1016 CELP v3.2
% COPYRIGHT (C) 1995-99 ANDREAS SPANIAS AND TED PAINTER
%
% This Copyright applies only to this particular MATLAB implementation
% of the FS-1016 CELP coder.  The MATLAB software is intended only for educational
% purposes.  No other use is intended or authorized.  This is not a public
% domain program and distribution to individuals or networks is strictly
% prohibited.  Be aware that use of the standard in any form is goverened
% by rules of the US DoD.  Therefore patents and royalties may apply to
% authors, companies, or committees associated with this standard, FS-1016.  For
% questions regarding the MATLAB implementation please contact Andreas
% Spanias at (480) 965-1837.  For questions on rules,
% royalties, or patents associated with the standard, please contact the DoD.
%
% ALL DERIVATIVE WORKS MUST INCLUDE THIS COPYRIGHT NOTICE.
%
% ******************************************************************
% SEGSNR
%
% MATLAB REVISION HISTORY
%
% PORTED TO MATLAB FROM CELP 3.2a C RELEASE
% 8-8-94
%
% ******************************************************************
%
% DESCRIPTION
%
% Calculate average segmental SNR
%
% DESIGN NOTES
%
% See any standard text for segmental SNR definition. This version
% runs SNR summations for entire input vector length.
%
% VARIABLES
%
% INPUTS
%   s          -     "Reference" speech input
%   ts         -     "Test" speech input
%   sumsnr     -     Average SNR accumulator
%   framesnr   -     Average SNR counter
%
% OUTPUTS
%   sumsnr     -     Average SNR accumulator
%   framesnr   -     Average SNR counter
%   snrflag    -     Flag for valid snr value
%   logsnr     -     Segmental SNR for current frame, log (dB) scale
%
% INTERNALS
%   seng       -     "Ref" energy
%   neng       -     Difference energy
%   snr        -     Segmental SNR for current frame, linear scale
%
% CONSTANTS
%   TRUE       -     Logical 1
%   FALSE      -     Logical 0
%
% ******************************************************************

function [ sumsnr, framesnr, snrflag, logsnr ] = segsnr( s, ts, sumsnr, framesnr )

% DELCARE GLOBAL CONSTANTS
global TRUE FALSE

% INIT RETURN VALUES
snrflag = FALSE;

% COMPUTE SEGMENTAL ENERGIES
seng = sum( s .* s );
neng = sum( ( s - ts ) .^ 2 );

% COMPUTE SEGMENTAL SNR AND UPDATE ACCUMULATORS
if neng > 0
    snr = seng / neng;
    if snr > 0.0
        % ONLY INCLUDE SEGMENTS WHERE -10 dB < snr < 64 dB
        logsnr = 10 * log10(snr);
        if ( logsnr > -10 ) & ( logsnr < 64 )
            snrflag = TRUE;
            sumsnr = sumsnr + snr;
            framesnr = framesnr + 1;
        end
    end
end

% Avoid unassigned return error
if snrflag==FALSE
   sumsnr=0;
   framesnr=0;
   logsnr=0;
end